@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
 // 'ADU' MP3 streams (for improved loss-tolerance)
 // Implementation
 
@@ -44,6 +44,7 @@ public:
   unsigned backpointer;
 
   struct timeval presentationTime;
+  unsigned durationInMicroseconds;
 };
 
 unsigned const Segment::headerSize = 4;
@@ -82,7 +83,9 @@ public:
 private:
   static void sqAfterGettingSegment(void* clientData,
 				    unsigned numBytesRead,
-				    struct timeval presentationTime);
+				    unsigned numTruncatedBytes,
+				    struct timeval presentationTime,
+				    unsigned durationInMicroseconds);
 
   Boolean sqAfterGettingCommon(Segment& seg, unsigned numBytesRead); 
   Boolean isEmptyOrFull() {return headIndex() == nextFreeIndex();}
@@ -181,6 +184,7 @@ Boolean ADUFromMP3Source::doGetNextFrame1() {
   // Output an ADU from the tail segment:
   fFrameSize = tailSeg->headerSize+tailSeg->sideInfoSize+tailSeg->aduSize;
   fPresentationTime = tailSeg->presentationTime;
+  fDurationInMicroseconds = tailSeg->durationInMicroseconds;
   unsigned descriptorSize
     = fIncludeADUdescriptors ? ADUdescriptor::computeSize(fFrameSize) : 0;
 #ifdef DEBUG
@@ -318,7 +322,8 @@ void MP3FromADUSource::doGetNextFrame() {
     fSegments->enqueueNewSegment(fInputSource, this);
   } else {
     // Return a frame now:
-    generateFrameFromHeadADU(); // sets fFrameSize and fPresentationTime
+    generateFrameFromHeadADU();
+        // sets fFrameSize, fPresentationTime, and fDurationInMicroseconds
 
     // Call our own 'after getting' function.  Because we're not a 'leaf'
     // source, we can call this directly, without risking infinite recursion.
@@ -412,6 +417,7 @@ Boolean MP3FromADUSource::generateFrameFromHeadADU() {
     // output header and side info:
     fFrameSize = seg->frameSize;
     fPresentationTime = seg->presentationTime;
+    fDurationInMicroseconds = seg->durationInMicroseconds;
     memmove(toPtr, seg->dataStart(), seg->headerSize + seg->sideInfoSize);
     toPtr += seg->headerSize + seg->sideInfoSize;
     
@@ -504,11 +510,14 @@ void SegmentQueue::enqueueNewSegment(FramedSource* inputSource,
 
 void SegmentQueue::sqAfterGettingSegment(void* clientData,
 					 unsigned numBytesRead,
-					 struct timeval presentationTime) {
+					 unsigned /*numTruncatedBytes*/,
+					 struct timeval presentationTime,
+					 unsigned durationInMicroseconds) {
   SegmentQueue* segQueue = (SegmentQueue*)clientData;
   Segment& seg = segQueue->nextFreeSegment();
 
   seg.presentationTime = presentationTime;
+  seg.durationInMicroseconds = durationInMicroseconds;
 
   if (segQueue->sqAfterGettingCommon(seg, numBytesRead)) {
 #ifdef DEBUG
