@@ -27,6 +27,8 @@ MPEG1or2DemuxedElementaryStream(UsageEnvironment& env, u_int8_t streamIdTag,
 			    MPEG1or2Demux& sourceDemux)
   : FramedSource(env),
     fOurStreamIdTag(streamIdTag), fOurSourceDemux(sourceDemux) {
+  lastSeenSCR.highBit = 0; lastSeenSCR.remainingBits = 0; lastSeenSCR.extension = 0;
+
   // Set our MIME type string for known media types:
   if ((streamIdTag&0xE0) == 0xC0) {
     fMIMEtype = "audio/mpeg";
@@ -68,9 +70,22 @@ void MPEG1or2DemuxedElementaryStream
 		    unsigned durationInMicroseconds) {
   MPEG1or2DemuxedElementaryStream* stream
     = (MPEG1or2DemuxedElementaryStream*)clientData;
-  stream->fFrameSize = frameSize;
-  stream->fNumTruncatedBytes = numTruncatedBytes;
-  stream->fPresentationTime = presentationTime;
-  stream->fDurationInMicroseconds = durationInMicroseconds;
-  FramedSource::afterGetting(stream);
+  stream->afterGettingFrame1(frameSize, numTruncatedBytes,
+			     presentationTime, durationInMicroseconds);
+}
+
+void MPEG1or2DemuxedElementaryStream
+::afterGettingFrame1(unsigned frameSize, unsigned numTruncatedBytes,
+		     struct timeval presentationTime,
+		     unsigned durationInMicroseconds) {
+  fFrameSize = frameSize;
+  fNumTruncatedBytes = numTruncatedBytes;
+  fPresentationTime = presentationTime;
+  fDurationInMicroseconds = durationInMicroseconds;
+
+  lastSeenSCR.highBit = fOurSourceDemux.lastSeenSCR.highBit;
+  lastSeenSCR.remainingBits = fOurSourceDemux.lastSeenSCR.remainingBits;
+  lastSeenSCR.extension = fOurSourceDemux.lastSeenSCR.extension;
+
+  FramedSource::afterGetting(this);
 }
