@@ -24,6 +24,8 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "MPEG1or2AudioRTPSink.hh"
 #include "MPEG1or2VideoStreamFramer.hh"
 #include "MPEG1or2VideoRTPSink.hh"
+#include "AC3AudioStreamFramer.hh"
+#include "AC3AudioRTPSink.hh"
 
 MPEG1or2DemuxedServerMediaSubsession* MPEG1or2DemuxedServerMediaSubsession
 ::createNew(MPEG1or2FileServerDemux& demux, u_int8_t streamIdTag,
@@ -51,13 +53,16 @@ FramedSource* MPEG1or2DemuxedServerMediaSubsession
     es = fOurDemux.newElementaryStream(clientSessionId, fStreamIdTag);
     if (es == NULL) break;
 
-    if ((fStreamIdTag&0xF0) == 0xC0 /*audio*/) {
+    if ((fStreamIdTag&0xF0) == 0xC0 /*MPEG audio*/) {
       estBitrate = 128; // kbps, estimate
       return MPEG1or2AudioStreamFramer::createNew(fOurDemux.envir(), es);
     } else if ((fStreamIdTag&0xF0) == 0xE0 /*video*/) {
       estBitrate = 500; // kbps, estimate
       return MPEG1or2VideoStreamFramer::createNew(fOurDemux.envir(), es,
 						  fIFramesOnly, fVSHPeriod);
+    } else if (fStreamIdTag == 0xBD /*AC-3 audio*/) {
+      estBitrate = 192; // kbps, estimate
+      return AC3AudioStreamFramer::createNew(fOurDemux.envir(), es);
     } else { // unknown stream type
       break;
     }
@@ -69,14 +74,19 @@ FramedSource* MPEG1or2DemuxedServerMediaSubsession
 }
 
 RTPSink* MPEG1or2DemuxedServerMediaSubsession
-::createNewRTPSink(Groupsock* rtpGroupsock, unsigned char /*rtpPayloadTypeIfDynamic*/,
-		   FramedSource* /*inputSource*/) {
-  if ((fStreamIdTag&0xF0) == 0xC0 /*audio*/) {
+::createNewRTPSink(Groupsock* rtpGroupsock, unsigned char rtpPayloadTypeIfDynamic,
+		   FramedSource* inputSource) {
+  if ((fStreamIdTag&0xF0) == 0xC0 /*MPEG audio*/) {
     return MPEG1or2AudioRTPSink::createNew(envir(), rtpGroupsock);
   } else if ((fStreamIdTag&0xF0) == 0xE0 /*video*/) {
     return MPEG1or2VideoRTPSink::createNew(envir(), rtpGroupsock);
+  } else if (fStreamIdTag == 0xBD /*AC-3 audio*/) {
+    // Get the sampling frequency from the audio source; use it for the RTP frequency:
+    AC3AudioStreamFramer* audioSource
+      = (AC3AudioStreamFramer*)inputSource;
+    return AC3AudioRTPSink::createNew(envir(), rtpGroupsock, rtpPayloadTypeIfDynamic,
+				      audioSource->samplingRate());
   } else {
     return NULL;
   }
 }
-
