@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2003 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an MPEG-4 video elementary stream into
 //   frames for:
 // - Visual Object Sequence (VS) Header + Visual Object (VO) Header
@@ -36,7 +36,8 @@ enum MPEGParseState {
   PARSING_VISUAL_OBJECT,
   PARSING_VIDEO_OBJECT_LAYER,
   PARSING_GROUP_OF_VIDEO_OBJECT_PLANE,
-  PARSING_VIDEO_OBJECT_PLANE
+  PARSING_VIDEO_OBJECT_PLANE,
+  PARSING_VISUAL_OBJECT_SEQUENCE_END_CODE
 }; 
 
 class MPEG4VideoStreamParser: public MPEGVideoStreamParser {
@@ -59,6 +60,7 @@ private:
   unsigned parseVideoObjectLayer();
   unsigned parseGroupOfVideoObjectPlane();
   unsigned parseVideoObjectPlane();
+  unsigned parseVisualObjectSequenceEndCode();
 
   // These are used for parsing within an already-read frame:
   Boolean getNextFrameBit(u_int8_t& result);
@@ -73,7 +75,10 @@ private:
   u_int32_t vop_time_increment_resolution;
   unsigned fNumVTIRBits;
       // # of bits needed to count to "vop_time_increment_resolution"
-  unsigned fSecondsSinceLastGOV, fTotalTicksSinceLastGOV, fPrevNewTotalTicks; 
+  u_int8_t fixed_vop_rate;
+  unsigned fixed_vop_time_increment; // used if 'fixed_vop_rate' is set
+  unsigned fSecondsSinceLastTimeCode, fTotalTicksSinceLastTimeCode, fPrevNewTotalTicks; 
+  unsigned fPrevPictureCountDelta;
 };
 
 
@@ -143,8 +148,9 @@ MPEG4VideoStreamParser
   : MPEGVideoStreamParser(usingSource, inputSource),
     fCurrentParseState(PARSING_VISUAL_OBJECT_SEQUENCE),
     vop_time_increment_resolution(0), fNumVTIRBits(0),
-    fSecondsSinceLastGOV(0), fTotalTicksSinceLastGOV(0),
-    fPrevNewTotalTicks((unsigned)(~0)) {
+    fixed_vop_rate(0), fixed_vop_time_increment(0),
+    fSecondsSinceLastTimeCode(0), fTotalTicksSinceLastTimeCode(0),
+    fPrevNewTotalTicks(0), fPrevPictureCountDelta(1) {
 }
 
 MPEG4VideoStreamParser::~MPEG4VideoStreamParser() {
@@ -176,13 +182,16 @@ unsigned MPEG4VideoStreamParser::parse() {
     case PARSING_VIDEO_OBJECT_PLANE: {
       return parseVideoObjectPlane();
     }
+    case PARSING_VISUAL_OBJECT_SEQUENCE_END_CODE: {
+      return parseVisualObjectSequenceEndCode();
+    }
     default: {
       return 0; // shouldn't happen
     }
     }
   } catch (int /*e*/) {
 #ifdef DEBUG
-    fprintf(stderr, "MPEG4VideoStreamParser::parse() EXCEPTION\n");
+    fprintf(stderr, "MPEG4VideoStreamParser::parse() EXCEPTION (This is normal behavior - *not* an error)\n");
 #endif
     return 0;  // the parsing got interrupted
   }
@@ -234,7 +243,7 @@ unsigned MPEG4VideoStreamParser
   setParseState(PARSING_VISUAL_OBJECT);
 
   // Compute this frame's presentation time:
-  usingSource()->computePresentationTime(fTotalTicksSinceLastGOV);
+  usingSource()->computePresentationTime(fTotalTicksSinceLastTimeCode);
 
   // This header forms part of the 'configuration' information:
   usingSource()->appendToNewConfig(fStartOfFrame, curFrameSize());
@@ -288,7 +297,7 @@ unsigned MPEG4VideoStreamParser::parseVisualObject() {
   setParseState(PARSING_VIDEO_OBJECT_LAYER);
 
   // Compute this frame's presentation time:
-  usingSource()->computePresentationTime(fTotalTicksSinceLastGOV);
+  usingSource()->computePresentationTime(fTotalTicksSinceLastTimeCode);
 
   // This header forms part of the 'configuration' information:
   usingSource()->appendToNewConfig(fStartOfFrame, curFrameSize());
@@ -370,26 +379,22 @@ void MPEG4VideoStreamParser::analyzeVOLHeader() {
       break;
     }
 
-    u_int8_t fixed_vop_rate;
     if (!getNextFrameBit(fixed_vop_rate)) break;
     if (fixed_vop_rate) {
-      // Get the following "fixed_vop_time_increment", and use this to set
-      // the frame rate:
-      u_int32_t fixed_vop_time_increment;
+      // Get the following "fixed_vop_time_increment":
       if (!getNextFrameBits(fNumVTIRBits, fixed_vop_time_increment)) break;
+#ifdef DEBUG
+      fprintf(stderr, "fixed_vop_time_increment: %d\n", fixed_vop_time_increment);
       if (fixed_vop_time_increment == 0) {
 	usingSource()->envir() << "MPEG4VideoStreamParser::analyzeVOLHeader(): fixed_vop_time_increment is zero!\n";
-	break;
       }
-      usingSource()->fFrameRate
-	= vop_time_increment_resolution/(double)fixed_vop_time_increment;
-    } else {
-      // Use "vop_time_increment_resolution" as the 'frame rate'
-      // (really, 'tick rate'):
-      usingSource()->fFrameRate = (double)vop_time_increment_resolution;
+#endif
     }
+    // Use "vop_time_increment_resolution" as the 'frame rate'
+    // (really, 'tick rate'):
+    usingSource()->fFrameRate = (double)vop_time_increment_resolution;
 #ifdef DEBUG
-    fprintf(stderr, "fixed_vop_rate: %d; 'frame' rate: %f\n", fixed_vop_rate, usingSource()->fFrameRate);
+    fprintf(stderr, "fixed_vop_rate: %d; 'frame' (really tick) rate: %f\n", fixed_vop_rate, usingSource()->fFrameRate);
 #endif
 
     return;
@@ -428,7 +433,7 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectLayer() {
 		: PARSING_VIDEO_OBJECT_PLANE);
 
   // Compute this frame's presentation time:
-  usingSource()->computePresentationTime(fTotalTicksSinceLastGOV);
+  usingSource()->computePresentationTime(fTotalTicksSinceLastTimeCode);
 
   // This header ends the 'configuration' information:
   usingSource()->appendToNewConfig(fStartOfFrame, curFrameSize());
@@ -467,13 +472,15 @@ unsigned MPEG4VideoStreamParser::parseGroupOfVideoObjectPlane() {
   }
   
   // Compute this frame's presentation time:
-  usingSource()->computePresentationTime(fTotalTicksSinceLastGOV);
+  usingSource()->computePresentationTime(fTotalTicksSinceLastTimeCode);
 
   // Record the time code:
   usingSource()->setTimeCode(time_code_hours, time_code_minutes,
-                             time_code_seconds, 0,
-			     fTotalTicksSinceLastGOV);
-  fSecondsSinceLastGOV = fTotalTicksSinceLastGOV = 0;
+                             time_code_seconds, 0, 0);
+    // Note: Because the GOV header can appear anywhere (not just at a 1s point), we
+    // don't pass "fTotalTicksSinceLastTimeCode" as the "picturesSinceLastGOP" parameter.
+  fSecondsSinceLastTimeCode = 0;
+  if (fixed_vop_rate) fTotalTicksSinceLastTimeCode = 0;
 
   setParseState(PARSING_VIDEO_OBJECT_PLANE);
 
@@ -482,16 +489,14 @@ unsigned MPEG4VideoStreamParser::parseGroupOfVideoObjectPlane() {
 
 unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
 #ifdef DEBUG
-  fprintf(stderr, "parsing VideoObjectPlane\n");
+  fprintf(stderr, "#parsing VideoObjectPlane\n");
 #endif
   // Note that we've already read the VOP_START_CODE
   save4Bytes(VOP_START_CODE);
 
   // Get the "vop_coding_type" from the next byte:
   u_int8_t nextByte = get1Byte(); saveByte(nextByte);
-#ifdef DEBUG
   u_int8_t vop_coding_type = nextByte>>6;
-#endif
 
   // Next, get the "modulo_time_base" by counting the '1' bits that follow.
   // We look at the next 32-bits only.  This should be enough in most cases.
@@ -533,20 +538,53 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
   saveToNextCode(next4Bytes);
   
   // Update our counters based on the frame timing information that we saw:
-  fSecondsSinceLastGOV += modulo_time_base;
-  unsigned newTotalTicks
-    = fSecondsSinceLastGOV*vop_time_increment_resolution
-    + vop_time_increment;
-  if (newTotalTicks == fPrevNewTotalTicks) {
-    // This is apparently a buggy MPEG-4 video stream, because
-    // "vop_time_increment" did not change.  Overcome this error,
-    // by pretending that it did change.
-    usingSource()->fPictureCount += vop_time_increment;
-    fTotalTicksSinceLastGOV += vop_time_increment;
+  if (fixed_vop_time_increment > 0) {
+    // This is a 'fixed_vop_rate' stream.  Use 'fixed_vop_time_increment':
+    usingSource()->fPictureCount += fixed_vop_time_increment;
+    if (vop_time_increment > 0 || modulo_time_base > 0) {
+      fTotalTicksSinceLastTimeCode += fixed_vop_time_increment;
+      // Note: "fSecondsSinceLastTimeCode" and "fPrevNewTotalTicks" are not used.
+    }
   } else {
-    fPrevNewTotalTicks = newTotalTicks;
-    usingSource()->fPictureCount += newTotalTicks - fTotalTicksSinceLastGOV;
-    fTotalTicksSinceLastGOV = newTotalTicks;
+    // Use 'vop_time_increment':
+    unsigned newTotalTicks
+      = (fSecondsSinceLastTimeCode + modulo_time_base)*vop_time_increment_resolution
+      + vop_time_increment;
+    if (newTotalTicks == fPrevNewTotalTicks && fPrevNewTotalTicks > 0) {
+      // This is apparently a buggy MPEG-4 video stream, because
+      // "vop_time_increment" did not change.  Overcome this error,
+      // by pretending that it did change.
+#ifdef DEBUG
+      fprintf(stderr, "Buggy MPEG-4 video stream: \"vop_time_increment\" did not change!\n");
+#endif
+      // The following assumes that we don't have 'B' frames.  If we do, then TARFU!
+      usingSource()->fPictureCount += vop_time_increment;
+      fTotalTicksSinceLastTimeCode += vop_time_increment;
+      fSecondsSinceLastTimeCode += modulo_time_base;
+    } else {
+      if (newTotalTicks < fPrevNewTotalTicks && vop_coding_type != 2/*B*/
+	  && modulo_time_base == 0 && vop_time_increment == 0) {
+	// This is another kind of buggy MPEG-4 video stream, in which
+	// "vop_time_increment" wraps around, but without
+	// "modulo_time_base" changing.  Overcome this by pretending that
+	// "vop_time_increment" really did wrap around:
+#ifdef DEBUG
+	fprintf(stderr, "Buggy MPEG-4 video stream: \"vop_time_increment\" wrapped around, but without \"modulo_time_base\" changing!\n");
+#endif
+	++fSecondsSinceLastTimeCode;
+	newTotalTicks += vop_time_increment_resolution;
+      }
+      fPrevNewTotalTicks = newTotalTicks;
+      int pictureCountDelta = newTotalTicks - fTotalTicksSinceLastTimeCode;
+      if (pictureCountDelta <= 0) pictureCountDelta = fPrevPictureCountDelta;
+          // ensures that the picture count always increases
+      usingSource()->fPictureCount += pictureCountDelta;
+      fPrevPictureCountDelta = pictureCountDelta;
+      fTotalTicksSinceLastTimeCode = newTotalTicks;
+      if (vop_coding_type != 2/*B*/) {
+	fSecondsSinceLastTimeCode += modulo_time_base;
+      }
+    }
   }
   
   // The next thing to parse depends on the code that we just saw,
@@ -554,7 +592,7 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
   usingSource()->fPictureEndMarker = True; // HACK #####
   switch (next4Bytes) {
   case VISUAL_OBJECT_SEQUENCE_END_CODE: {
-    setParseState(PARSING_VISUAL_OBJECT_SEQUENCE);
+    setParseState(PARSING_VISUAL_OBJECT_SEQUENCE_END_CODE);
     break;
   }
   case VISUAL_OBJECT_SEQUENCE_START_CODE: {
@@ -578,7 +616,22 @@ unsigned MPEG4VideoStreamParser::parseVideoObjectPlane() {
   }
 
   // Compute this frame's presentation time:
-  usingSource()->computePresentationTime(fTotalTicksSinceLastGOV);
+  usingSource()->computePresentationTime(fTotalTicksSinceLastTimeCode);
+
+  return curFrameSize();
+}
+
+unsigned MPEG4VideoStreamParser::parseVisualObjectSequenceEndCode() {
+#ifdef DEBUG
+  fprintf(stderr, "parsing VISUAL_OBJECT_SEQUENCE_END_CODE\n");
+#endif
+  // Note that we've already read the VISUAL_OBJECT_SEQUENCE_END_CODE
+  save4Bytes(VISUAL_OBJECT_SEQUENCE_END_CODE);
+
+  setParseState(PARSING_VISUAL_OBJECT_SEQUENCE);
+
+  // Treat this as if we had ended a picture:
+  usingSource()->fPictureEndMarker = True; // HACK #####
 
   return curFrameSize();
 }
