@@ -20,6 +20,10 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 // Implementation
 
 #include "liveMedia.hh"
+#ifdef SUPPORT_REAL_RTSP
+#include "../RealRTSP/include/RealRDTSource.hh"
+#include "../RealRTSP/include/RealSDP.hh"
+#endif
 #include "GroupsockHelper.hh"
 #include <ctype.h>
 
@@ -56,8 +60,12 @@ Boolean MediaSession::lookupByName(UsageEnvironment& env,
 }
 
 MediaSession::MediaSession(UsageEnvironment& env)
-  : Medium(env), fSubsessionsHead(NULL), fSubsessionsTail(NULL),
+  : Medium(env),
+    fSubsessionsHead(NULL), fSubsessionsTail(NULL),
     fConnectionEndpointName(NULL), fMaxPlayEndTime(0.0) {
+#ifdef SUPPORT_REAL_RTSP
+  RealInitSDPAttributes(this);
+#endif
   fSourceFilterAddr.s_addr = 0;
 
   // Get our host name, and use this for the RTCP CNAME:
@@ -77,6 +85,9 @@ MediaSession::~MediaSession() {
   delete fSubsessionsHead;
   delete[] fCNAME;
   delete[] fConnectionEndpointName;
+#ifdef SUPPORT_REAL_RTSP
+  RealReclaimSDPAttributes(this);
+#endif
 }
 
 Boolean MediaSession::isMediaSession() const {
@@ -102,6 +113,9 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
     if (parseSDPLine_c(sdpLine)) continue;
     if (parseSDPAttribute_range(sdpLine)) continue;
     if (parseSDPAttribute_source_filter(sdpLine)) continue;
+#ifdef SUPPORT_REAL_RTSP
+    if (RealParseSDPAttributes(this, sdpLine)) continue;
+#endif
   }
     
   while (sdpLine != NULL) {
@@ -166,6 +180,9 @@ Boolean MediaSession::initializeWithSDP(char const* sdpDescription) {
       if (subsession->parseSDPAttribute_x_mct_slap(sdpLine)) continue;
       if (subsession->parseSDPAttribute_x_dimensions(sdpLine)) continue;
       if (subsession->parseSDPAttribute_x_framerate(sdpLine)) continue;
+#ifdef SUPPORT_REAL_RTSP
+      if (RealParseSDPAttributes(subsession, sdpLine)) continue;
+#endif
 
       // (Later, check for malformed lines, and other valid SDP lines#####)
     }
@@ -475,10 +492,10 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fRTPTimestampFrequency(0), fControlPath(NULL),
     fSourceFilterAddr(parent.sourceFilterAddr()),
     fAuxiliarydatasizelength(0), fConstantduration(0), fConstantsize(0),
-    fCtsdeltalength(0), fDe_interleavebuffersize(0), fDtsdeltalength(0),
-    fIndexdeltalength(0), fIndexlength(0), fMaxdisplacement(0),
-    fObjecttype(0), fProfile_level_id(0), fSizelength(0),
-    fStreamstateindication(0), fStreamtype(0),
+    fCRC(0), fCtsdeltalength(0), fDe_interleavebuffersize(0), fDtsdeltalength(0),
+    fIndexdeltalength(0), fIndexlength(0), fInterleaving(0), fMaxdisplacement(0),
+    fObjecttype(0), fOctetalign(0), fProfile_level_id(0), fRobustsorting(0),
+    fSizelength(0), fStreamstateindication(0), fStreamtype(0),
     fCpresent(False), fRandomaccessindication(False),
     fConfig(NULL), fMode(NULL),
     fPlayEndTime(0.0),
@@ -486,6 +503,9 @@ MediaSubsession::MediaSubsession(MediaSession& parent)
     fVideoWidth(0), fVideoHeight(0), fVideoFPS(0), fNumChannels(1),
     fRTPSocket(NULL), fRTCPSocket(NULL),
     fRTPSource(NULL), fRTCPInstance(NULL), fReadSource(NULL) {
+#ifdef SUPPORT_REAL_RTSP
+    RealInitSDPAttributes(this);
+#endif
 }
 
 MediaSubsession::~MediaSubsession() {
@@ -496,6 +516,9 @@ MediaSubsession::~MediaSubsession() {
   delete[] fControlPath; delete[] fConfig; delete[] fMode;
 
   delete fNext;
+#ifdef SUPPORT_REAL_RTSP
+  RealReclaimSDPAttributes(this);
+#endif
 }
 
 float MediaSubsession::playEndTime() const {
@@ -584,6 +607,20 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
 				       fRTPPayloadFormat,
 				       fRTPTimestampFrequency);
           // Note that fReadSource will differ from fRTPSource in this case
+    } else if (strcmp(fCodecName, "AMR") == 0) { // AMR audio (narrowband)
+      fReadSource =
+	AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
+				     fRTPPayloadFormat, 0 /*isWideband*/,
+				     fNumChannels, fOctetalign, fInterleaving,
+				     fRobustsorting, fCRC);
+          // Note that fReadSource will differ from fRTPSource in this case
+    } else if (strcmp(fCodecName, "AMR-WB") == 0) { // AMR audio (wideband)
+      fReadSource =
+	AMRAudioRTPSource::createNew(env(), fRTPSocket, fRTPSource,
+				     fRTPPayloadFormat, 1 /*isWideband*/,
+				     fNumChannels, fOctetalign, fInterleaving,
+				     fRobustsorting, fCRC);
+          // Note that fReadSource will differ from fRTPSource in this case
     } else if (strcmp(fCodecName, "MPA") == 0) { // MPEG-1 or 2 audio
       fReadSource = fRTPSource
 	= MPEG1or2AudioRTPSource::createNew(env(), fRTPSocket,
@@ -674,6 +711,13 @@ Boolean MediaSubsession::initiate(int useSpecialRTPoffset) {
       // A UDP-packetized text stream (*not* a RTP stream)
       fReadSource = BasicUDPSource::createNew(env(), fRTPSocket);
       fRTPSource = NULL; // Note!
+#ifdef SUPPORT_REAL_RTSP
+    } else if (strcmp(fCodecName, "X-PN-REALAUDIO") == 0 ||
+	       strcmp(fCodecName, "X-PN-REALVIDEO") == 0) {
+      // A RealNetworks 'RDT' stream (*not* a RTP stream)
+      fReadSource = RealRDTSource::createNew(env());
+      fRTPSource = NULL; // Note!
+#endif
     } else if (  strcmp(fCodecName, "PCMU") == 0 // PCM u-law audio
 	       || strcmp(fCodecName, "GSM") == 0 // GSM audio
 	       || strcmp(fCodecName, "PCMA") == 0 // PCM a-law audio
@@ -906,6 +950,8 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	fConstantduration = u;
       } else if (sscanf(line, " constantsize; = %u", &u) == 1) {
 	fConstantsize = u;
+      } else if (sscanf(line, " crc = %u", &u) == 1) {
+	fCRC = u;
       } else if (sscanf(line, " ctsdeltalength = %u", &u) == 1) {
 	fCtsdeltalength = u;
       } else if (sscanf(line, " de-interleavebuffersize = %u", &u) == 1) {
@@ -916,12 +962,18 @@ Boolean MediaSubsession::parseSDPAttribute_fmtp(char const* sdpLine) {
 	fIndexdeltalength = u;
       } else if (sscanf(line, " indexlength = %u", &u) == 1) {
 	fIndexlength = u;
+      } else if (sscanf(line, " interleaving = %u", &u) == 1) {
+	fInterleaving = u;
       } else if (sscanf(line, " maxdisplacement = %u", &u) == 1) {
 	fMaxdisplacement = u;
       } else if (sscanf(line, " objecttype = %u", &u) == 1) {
 	fObjecttype = u;
+      } else if (sscanf(line, " octet-align = %u", &u) == 1) {
+	fOctetalign = u;
       } else if (sscanf(line, " profile-level-id = %u", &u) == 1) {
 	fProfile_level_id = u;
+      } else if (sscanf(line, " robust-sorting = %u", &u) == 1) {
+	fRobustsorting = u;
       } else if (sscanf(line, " sizelength = %u", &u) == 1) {
 	fSizelength = u;
       } else if (sscanf(line, " streamstateindication = %u", &u) == 1) {
