@@ -226,7 +226,7 @@ void MultiFramedRTPSource::networkReadHandler(MultiFramedRTPSource* source,
     // Skip over any CSRC identifiers in the header:
     unsigned cc = (rtpHdr>>24)&0xF;
     if (bPacket->dataSize() < cc) break;
-    ADVANCE(cc);
+    ADVANCE(cc*4);
     
     // Check for (& ignore) any RTP header extension
     if (rtpHdr&0x10000000) {
@@ -345,6 +345,12 @@ void BufferedPacket::removePadding(unsigned numBytes) {
   fTail -= numBytes;
 }
 
+void BufferedPacket::appendData(unsigned char* newData, unsigned numBytes) {
+  if (numBytes > fPacketSize-fTail) numBytes = fPacketSize - fTail;
+  memmove(&fBuf[fTail], newData, numBytes); 
+  fTail += numBytes;
+}
+
 void BufferedPacket::use(unsigned char* to, unsigned toSize,
 			 unsigned& bytesUsed, unsigned& bytesTruncated,
 			 unsigned short& rtpSeqNo, unsigned& rtpTimestamp,
@@ -391,7 +397,11 @@ ReorderingPacketBuffer
 }
 
 ReorderingPacketBuffer::~ReorderingPacketBuffer() {
-  delete fHeadPacket;
+  if (fHeadPacket == NULL) {
+    delete fSavedPacket;
+  } else {
+    delete fHeadPacket; // will also delete fSavedPacket, because it's on the list
+  }
   delete fPacketFactory;
 }
 
