@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2002 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2004 Live Networks, Inc.  All rights reserved.
 // A sink that generates a QuickTime file from a composite media session
 // Implementation
 
@@ -382,7 +382,9 @@ Boolean QuickTimeFileSink::continuePlaying() {
 
 void QuickTimeFileSink
 ::afterGettingFrame(void* clientData, unsigned packetDataSize,
-		    struct timeval presentationTime) {
+		    unsigned /*numTruncatedBytes*/,
+		    struct timeval presentationTime,
+		    unsigned /*durationInMicroseconds*/) {
   SubsessionIOState* ioState = (SubsessionIOState*)clientData;
   if (!ioState->syncOK(presentationTime)) {
     // Ignore this data:
@@ -606,7 +608,8 @@ Boolean SubsessionIOState::setQTstate() {
       if (strcmp(fOurSubsession.codecName(), "X-QT") == 0 ||
 	  strcmp(fOurSubsession.codecName(), "X-QUICKTIME") == 0) {
 	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_genericMedia;
-      } else if (strcmp(fOurSubsession.codecName(), "H263-1998") == 0) {
+      } else if (strcmp(fOurSubsession.codecName(), "H263-1998") == 0 ||
+		 strcmp(fOurSubsession.codecName(), "H263-2000") == 0) {
 	fQTMediaDataAtomCreator = &QuickTimeFileSink::addAtom_h263;
 	fQTTimeScale = 600;
 	fQTTimeUnitsPerSample = fQTTimeScale/fOurSink.fMovieFPS;
