@@ -21,6 +21,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 #include "RTPSink.hh"
 #include "GroupsockHelper.hh"
 
+//#define DEBUG_RR 1 //#####@@@@@
 ////////// RTPSink //////////
 
 Boolean RTPSink::lookupByName(UsageEnvironment& env, char const* sinkName,
@@ -185,8 +186,9 @@ void RTPTransmissionStatsDB::add(unsigned SSRC, RTPTransmissionStats* stats) {
 ////////// RTPTransmissionStats //////////
 
 RTPTransmissionStats::RTPTransmissionStats(RTPSink& rtpSink, unsigned SSRC)
-  : fOurRTPSink(rtpSink), fSSRC(SSRC), fLastPacketNumReceived(0), fPacketLossRatio(0),
-    fTotNumPacketsLost(0), fJitter(0), fLastSRTime(0), fDiffSR_RRTime(0),
+  : fOurRTPSink(rtpSink), fSSRC(SSRC), fLastPacketNumReceived(0),
+    fPacketLossRatio(0), fTotNumPacketsLost(0), fJitter(0),
+    fLastSRTime(0), fDiffSR_RRTime(0),
     fFirstPacket(True) {
 }
 
@@ -203,12 +205,7 @@ void RTPTransmissionStats::noteIncomingRR(unsigned lossStats,
   } else {
     fOldValid = True;
     fOldLastPacketNumReceived = fLastPacketNumReceived;
-    fOldPacketLossRatio = fPacketLossRatio;
     fOldTotNumPacketsLost = fTotNumPacketsLost;
-    fOldJitter = fJitter;
-    fOldLastSRTime = fLastSRTime;
-    fOldDiffSR_RRTime = fDiffSR_RRTime;
-    fOldTimeReceived = fTimeReceived;
   }
   gettimeofday(&fTimeReceived, &Idunno);
 
@@ -253,18 +250,6 @@ unsigned RTPTransmissionStats::packetsReceivedSinceLastRR() const {
   return fLastPacketNumReceived-fOldLastPacketNumReceived;
 }
 
-u_int8_t RTPTransmissionStats::packetLossRatio() const {
-  if (!fOldValid) return 0;
-
-  return fPacketLossRatio;
-}
-
-unsigned RTPTransmissionStats::timeDiffBetweenRR() const {
-  if (!fOldValid) return 0;
-
-  return (fLastSRTime + fDiffSR_RRTime) - (fOldLastSRTime + fOldDiffSR_RRTime);
-}
-
 int RTPTransmissionStats::packetsLostBetweenRR() const {
   if (!fOldValid) return 0;
 
