@@ -73,7 +73,7 @@ void RTPSource::getAttributes() const {
 
 RTPReceptionStatsDB::RTPReceptionStatsDB(RTPSource& rtpSource)
   : fOurRTPSource(rtpSource),
-    fTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
+    fTable(HashTable::create(ONE_WORD_HASH_KEYS)), fTotNumPacketsReceived(0) {
   reset();
 }
 
@@ -105,6 +105,7 @@ void RTPReceptionStatsDB
 		     struct timeval& resultPresentationTime,
 		     Boolean& resultHasBeenSyncedUsingRTCP,
 		     unsigned packetSize) {
+  ++fTotNumPacketsReceived;
   RTPReceptionStats* stats = lookup(SSRC);
   if (stats == NULL) {
     // This is the first time we've heard from this SSRC.
