@@ -20,6 +20,10 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 #include "RTSPClient.hh"
 #include "GroupsockHelper.hh"
+#ifdef SUPPORT_REAL_RTSP
+#include "../RealRTSP/include/RealChallengeResponse.hh"
+#include "../RealRTSP/include/RealRDTSource.hh"
+#endif
 #include "our_md5.h"
 #include <stdlib.h>
 
@@ -59,7 +63,11 @@ RTSPClient::RTSPClient(UsageEnvironment& env,
   : Medium(env),
     fVerbosityLevel(verbosityLevel), fSocketNum(-1), fServerAddress(0),
     fCSeq(0), fBaseURL(NULL), fCurrentAuthenticator(NULL),
-    fTCPStreamIdCount(0), fLastSessionId(NULL) {
+    fTCPStreamIdCount(0), fLastSessionId(NULL)
+#ifdef SUPPORT_REAL_RTSP
+  , fRealChallengeStr(NULL)
+#endif
+{
   // Set the "User-Agent:" header to use in each request:
   char const* const libName = "LIVE.COM Streaming Media v";
   char const* const libVersionStr = LIVEMEDIA_LIBRARY_VERSION_STRING;
@@ -100,6 +108,9 @@ void RTSPClient::reset() {
 
   resetCurrentAuthenticator();
 
+#ifdef SUPPORT_REAL_RTSP
+  delete[] fRealChallengeStr; fRealChallengeStr = NULL;
+#endif
   delete[] fLastSessionId; fLastSessionId = NULL;
 }
 
@@ -144,12 +155,21 @@ char* RTSPClient::describeURL(char const* url, AuthRecord* authenticator) {
       "DESCRIBE %s RTSP/1.0\r\n"
       "CSeq: %d\r\n"
       "Accept: application/sdp\r\n"
-      // Uncomment the following to simulate a RealNetworks client
+#ifdef SUPPORT_REAL_RTSP
+      // Simulate a RealNetworks client
       // (RealNetworks' servers sometimes behave differently if they
       //  think the client is one of theirs.):
-      //"ClientID: WinNT_4.0_6.0.11.818_RealPlayer_RN10PD_en-us_686\r\n"
+      "Bandwidth: 10485800\r\n"
+      "GUID: 00000000-0000-0000-0000-000000000000\r\n"
+      "RegionData: 0\r\n"
+      "ClientID: Linux_2.4_6.0.9.1235_play32_RN01_EN_586\r\n"
+      "SupportsMaximumASMBandwidth: 1\r\n"
+      "Language: en-US\r\n"
+      "Require: com.real.retain-entity-for-setup\r\n"
+#endif
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
     unsigned cmdSize = strlen(cmdFmt)
       + strlen(url)
       + 20 /* max int len */
@@ -171,8 +191,8 @@ char* RTSPClient::describeURL(char const* url, AuthRecord* authenticator) {
     // Get the response from the server:
     unsigned const readBufSize = 20000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received DESCRIBE response: " << readBuf << "\n";
     }
@@ -378,7 +398,20 @@ char* RTSPClient::sendOptionsCmd(char const* url) {
     char* const cmdFmt =
       "OPTIONS * RTSP/1.0\r\n"
       "CSeq: %d\r\n"
-      "%s\r\n";
+#ifdef SUPPORT_REAL_RTSP
+      // Simulate a RealNetworks client
+      // (RealNetworks' servers sometimes behave differently if they
+      //  think the client is one of theirs.):
+      "User-Agent: RealMedia Player Version 6.0.9.1235 (linux-2.0-libc6-i386-gcc2.95)\r\n"
+      "ClientChallenge: 9e26d33f2984236010ef6253fb1887f7\r\n"
+      "PlayerStarttime: [28/03/2003:22:50:23 00:00]\r\n"
+      "CompanyID: KnKV4M4I/B2FjJ1TToLycw==\r\n"
+      "GUID: 00000000-0000-0000-0000-000000000000\r\n"
+      "RegionData: 0\r\n"
+      "ClientID: Linux_2.4_6.0.9.1235_play32_RN01_EN_586\r\n"
+#endif
+      "%s"
+      "\r\n";
     unsigned cmdSize = strlen(cmdFmt)
       + 20 /* max int len */
       + fUserAgentHeaderStrSize;
@@ -395,8 +428,8 @@ char* RTSPClient::sendOptionsCmd(char const* url) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received OPTIONS response: " << readBuf << "\n";
     }
@@ -420,8 +453,11 @@ char* RTSPClient::sendOptionsCmd(char const* url) {
       nextLineStart = getLine(lineStart);
 
       if (_strncasecmp(lineStart, "Public: ", 8) == 0) {
-	result = strDup(&lineStart[8]);
-	break;
+	delete[] result; result = strDup(&lineStart[8]);
+#ifdef SUPPORT_REAL_RTSP
+      } else if (_strncasecmp(lineStart, "RealChallenge1: ", 16) == 0) {
+	delete[] fRealChallengeStr; fRealChallengeStr = strDup(&lineStart[16]);
+#endif
       }
     }
   } while (0);
@@ -516,8 +552,8 @@ Boolean RTSPClient::announceSDPDescription(char const* url,
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received ANNOUNCE response: " << readBuf << "\n";
     }
@@ -649,14 +685,36 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
       sessionStr = "";
     }
 
+#ifdef SUPPORT_REAL_RTSP
+    char realBuf[200];
+    if (fRealChallengeStr != NULL) {
+      char challenge2[64];
+      char checksum[34];
+      real_calc_response_and_checksum(challenge2, checksum, fRealChallengeStr);
+      sprintf(realBuf, "RealChallenge2: %s, sd=%s\r\n", challenge2, checksum);
+    } else {
+      realBuf[0] = '\0';
+    }
+
+    // Tell the RDT source to use the RTSP TCP socket:
+    // ##### This assumes that the source is RDT; fix this later for other sources #####@@@@@
+    RealRDTSource* rdtSource = (RealRDTSource*)(subsession.readSource());
+    rdtSource->setInputSocket(fSocketNum);
+#endif
     // (Later implement more, as specified in the RTSP spec, sec D.1 #####)
     char* const cmdFmt =
       "SETUP %s%s%s RTSP/1.0\r\n"
       "CSeq: %d\r\n"
+#ifdef SUPPORT_REAL_RTSP
+      "%s"
+      "Transport: x-pn-tng/tcp;mode=play,rtp/avp/unicast;mode=play\r\n"
+#else
       "Transport: RTP/AVP%s%s%s=%d-%d\r\n"
+#endif
       "%s"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     char const *prefix, *separator, *suffix;
     constructSubsessionURL(subsession, prefix, separator, suffix);
@@ -695,8 +753,12 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
     sprintf(cmd, cmdFmt,
 	    prefix, separator, suffix,
 	    ++fCSeq,
+#ifdef SUPPORT_REAL_RTSP
+	    realBuf,
+#else
 	    transportTypeString, modeString, portTypeString,
 	        rtpNumber, rtcpNumber,
+#endif
 	    sessionStr,
 	    authenticatorStr,
 	    fUserAgentHeaderStr);
@@ -712,8 +774,8 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received SETUP response: " << readBuf << "\n";
     }
@@ -769,10 +831,12 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
     if (streamUsingTCP) {
       // Tell the subsession to receive RTP (and send/receive RTCP)
       // over the RTSP stream:
-      subsession.rtpSource()->setStreamSocket(fSocketNum,
-					      subsession.rtpChannelId);
-      subsession.rtcpInstance()->setStreamSocket(fSocketNum,
-						 subsession.rtcpChannelId);
+      if (subsession.rtpSource() != NULL)
+	subsession.rtpSource()->setStreamSocket(fSocketNum,
+						subsession.rtpChannelId);
+      if (subsession.rtcpInstance() != NULL)
+	subsession.rtcpInstance()->setStreamSocket(fSocketNum,
+						   subsession.rtcpChannelId);
     } else {
       // Normal case.
       // Set the RTP and RTCP sockets' destination address and port
@@ -789,6 +853,11 @@ Boolean RTSPClient::setupMediaSubsession(MediaSubsession& subsession,
 }
 
 Boolean RTSPClient::playMediaSession(MediaSession& session) {
+#ifdef SUPPORT_REAL_RTSP
+  // Set the "Subscribe" parameter before proceeding:
+  setMediaSessionParameter(session, "Subscribe", "stream=0;rule=8,stream=0;rule=9");
+  //#####@@@@@  stream=0;rule=0,stream=0;rule=1
+#endif
   char* cmd = NULL;
   do {
     // First, make sure that we have a RTSP session in progress
@@ -809,7 +878,8 @@ Boolean RTSPClient::playMediaSession(MediaSession& session) {
       "Session: %s\r\n"
       "Range: npt=0-\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     unsigned cmdSize = strlen(cmdFmt)
       + strlen(fBaseURL)
@@ -834,8 +904,8 @@ Boolean RTSPClient::playMediaSession(MediaSession& session) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received PLAY response: " << readBuf << "\n";
     }
@@ -882,7 +952,8 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
       "Session: %s\r\n"
       "Range: npt=%s-%s\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     char startStr[30], endStr[30];
     sprintf(startStr, "%.3f", start); sprintf(endStr, "%.3f", end);
@@ -924,8 +995,8 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received PLAY response: " << readBuf << "\n";
     }
@@ -966,6 +1037,78 @@ Boolean RTSPClient::playMediaSubsession(MediaSubsession& subsession,
   return False;
 }
 
+Boolean RTSPClient::pauseMediaSession(MediaSession& session) {
+  char* cmd = NULL;
+  do {
+    // First, make sure that we have a RTSP session in progress
+    if (fLastSessionId == NULL) {
+      envir().setResultMsg("No RTSP session is currently in progress\n");
+      break;
+    }
+
+    // Send the PAUSE command:
+
+    // First, construct an authenticator string:
+    char* authenticatorStr
+      = createAuthenticatorString(fCurrentAuthenticator, "PAUSE", fBaseURL);
+
+    char* const cmdFmt =
+      "PAUSE %s RTSP/1.0\r\n"
+      "CSeq: %d\r\n"
+      "Session: %s\r\n"
+      "Range: npt=0-\r\n"
+      "%s"
+      "%s"
+      "\r\n";
+
+    unsigned cmdSize = strlen(cmdFmt)
+      + strlen(fBaseURL)
+      + 20 /* max int len */
+      + strlen(fLastSessionId)
+      + strlen(authenticatorStr)
+      + fUserAgentHeaderStrSize;
+    cmd = new char[cmdSize];
+    sprintf(cmd, cmdFmt,
+	    fBaseURL,
+	    ++fCSeq,
+	    fLastSessionId,
+	    authenticatorStr,
+	    fUserAgentHeaderStr);
+    delete[] authenticatorStr;
+
+    if (!sendRequest(cmd)) {
+      envir().setResultErrMsg("PAUSE send() failed: ");
+      break;
+    }
+
+    // Get the response from the server:
+    unsigned const readBufSize = 10000;
+    char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
+    if (fVerbosityLevel >= 1) {
+      envir() << "Received PAUSE response: " << readBuf << "\n";
+    }
+
+    // Inspect the first line to check whether it's a result code 200
+    char* firstLine = readBuf;
+    /*char* nextLineStart =*/ getLine(firstLine);
+    unsigned responseCode;
+    if (!parseResponseCode(firstLine, responseCode)) break;
+    if (responseCode != 200) {
+      envir().setResultMsg("cannot handle PAUSE response: ", firstLine);
+      break;
+    }
+    // (Later, check "CSeq" too #####)
+
+    delete[] cmd;
+    return True;
+  } while (0);
+
+delete[] cmd;
+  return False;
+}
+
 Boolean RTSPClient::pauseMediaSubsession(MediaSubsession& subsession) {
   char* cmd = NULL;
   do {
@@ -986,7 +1129,8 @@ Boolean RTSPClient::pauseMediaSubsession(MediaSubsession& subsession) {
       "CSeq: %d\r\n"
       "Session: %s\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
     
     char const *prefix, *separator, *suffix;
     constructSubsessionURL(subsession, prefix, separator, suffix);
@@ -1013,8 +1157,8 @@ Boolean RTSPClient::pauseMediaSubsession(MediaSubsession& subsession) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received PAUSE response: " << readBuf << "\n";
     }
@@ -1060,7 +1204,8 @@ Boolean RTSPClient::recordMediaSubsession(MediaSubsession& subsession) {
       "Session: %s\r\n"
       "Range: npt=0-\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     char const *prefix, *separator, *suffix;
     constructSubsessionURL(subsession, prefix, separator, suffix);
@@ -1088,8 +1233,8 @@ Boolean RTSPClient::recordMediaSubsession(MediaSubsession& subsession) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received RECORD response: " << readBuf << "\n";
     }
@@ -1113,6 +1258,83 @@ Boolean RTSPClient::recordMediaSubsession(MediaSubsession& subsession) {
   return False;
 }
 
+Boolean RTSPClient::setMediaSessionParameter(MediaSession& session,
+					     char const* parameterName,
+					     char const* parameterValue) {
+  char* cmd = NULL;
+  do {
+    // First, make sure that we have a RTSP session in progreee
+    if (fLastSessionId == NULL) {
+      envir().setResultMsg("No RTSP session is currently in progress\n");
+      break;
+    }
+
+    // Send the SET_PARAMETER command:
+
+    // First, construct an authenticator string:
+    char* authenticatorStr
+      = createAuthenticatorString(fCurrentAuthenticator,
+				  "SET_PARAMETER", fBaseURL);
+
+    char* const cmdFmt =
+      "SET_PARAMETER %s RTSP/1.0\r\n"
+      "CSeq: %d\r\n"
+      "Session: %s\r\n"
+      "%s"
+      "%s"
+      "%s: %s\r\n"
+      "\r\n";
+
+    unsigned cmdSize = strlen(cmdFmt)
+      + strlen(fBaseURL)
+      + 20 /* max int len */
+      + strlen(fLastSessionId)
+      + strlen(authenticatorStr)
+      + fUserAgentHeaderStrSize
+      + strlen(parameterName) + strlen(parameterValue);
+    cmd = new char[cmdSize];
+    sprintf(cmd, cmdFmt,
+	    fBaseURL,
+	    ++fCSeq,
+	    fLastSessionId,
+	    authenticatorStr,
+	    fUserAgentHeaderStr,
+	    parameterName, parameterValue);
+    delete[] authenticatorStr;
+
+    if (!sendRequest(cmd)) {
+      envir().setResultErrMsg("SET_PARAMETER send() failed: ");
+      break;
+    }
+
+    // Get the response from the server:
+    unsigned const readBufSize = 10000;
+    char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
+    if (fVerbosityLevel >= 1) {
+      envir() << "Received SET_PARAMETER response: " << readBuf << "\n";
+    }
+
+    // Inspect the first line to check whether it's a result code 200
+    char* firstLine = readBuf;
+    /*char* nextLineStart =*/ getLine(firstLine);
+    unsigned responseCode;
+    if (!parseResponseCode(firstLine, responseCode)) break;
+    if (responseCode != 200) {
+      envir().setResultMsg("cannot handle SET_PARAMETER response: ", firstLine);
+      break;
+    }
+    // (Later, check "CSeq" too #####)
+
+    delete[] cmd;
+    return True;
+  } while (0);
+
+  delete[] cmd;
+  return False;
+}
+
 Boolean RTSPClient::teardownMediaSession(MediaSession& session) {
   char* cmd = NULL;
   do {
@@ -1134,7 +1356,8 @@ Boolean RTSPClient::teardownMediaSession(MediaSession& session) {
       "CSeq: %d\r\n"
       "Session: %s\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     unsigned cmdSize = strlen(cmdFmt)
       + strlen(fBaseURL)
@@ -1159,8 +1382,8 @@ Boolean RTSPClient::teardownMediaSession(MediaSession& session) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received TEARDOWN response: " << readBuf << "\n";
     }
@@ -1208,7 +1431,8 @@ Boolean RTSPClient::teardownMediaSubsession(MediaSubsession& subsession) {
       "CSeq: %d\r\n"
       "Session: %s\r\n"
       "%s"
-      "%s\r\n";
+      "%s"
+      "\r\n";
 
     char const *prefix, *separator, *suffix;
     constructSubsessionURL(subsession, prefix, separator, suffix);
@@ -1236,8 +1460,8 @@ Boolean RTSPClient::teardownMediaSubsession(MediaSubsession& subsession) {
     // Get the response from the server:
     unsigned const readBufSize = 10000;
     char readBuffer[readBufSize+1]; char* readBuf = readBuffer;
-    int bytesRead = getResponse(readBuf, readBufSize);
-    if (bytesRead < 0) break;
+    unsigned bytesRead = getResponse(readBuf, readBufSize);
+    if (bytesRead == 0) break;
     if (fVerbosityLevel >= 1) {
       envir() << "Received TEARDOWN response: " << readBuf << "\n";
     }
@@ -1467,8 +1691,8 @@ Boolean RTSPClient::sendRequest(char const* requestString) {
   return send(fSocketNum, requestString, strlen(requestString), 0) >= 0;
 }
 
-int RTSPClient::getResponse(char*& responseBuffer,
-			    unsigned responseBufferSize) {
+unsigned RTSPClient::getResponse(char*& responseBuffer,
+				 unsigned responseBufferSize) {
   struct sockaddr_in fromAddress;
   
   if (responseBufferSize == 0) return 0; // just in case...
@@ -1494,7 +1718,7 @@ int RTSPClient::getResponse(char*& responseBuffer,
 	  != 1) break;
 
       unsigned short size;
-      if (readSocket(envir(), fSocketNum, (unsigned char*)&size, 2,
+      if (readSocketExact(envir(), fSocketNum, (unsigned char*)&size, 2,
 		     fromAddress) != 2) break;
       size = ntohs(size);
       if (fVerbosityLevel >= 1) {
