@@ -13,13 +13,16 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2003, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2004, Live Networks, Inc.  All rights reserved
 // A common framework, used for the "openRTSP" and "playSIP" applications
 // Implementation
 
 #include "playCommon.hh"
 #include "BasicUsageEnvironment.hh"
 #include "GroupsockHelper.hh"
+#ifdef SUPPORT_REAL_RTSP
+#include "../RealRTSP/include/RealRMFF.hh"
+#endif
 
 #if defined(__WIN32__) || defined(_WIN32)
 #define snprintf _snprintf
@@ -40,6 +43,7 @@ void sessionTimerHandler(void* clientData);
 void shutdown(int exitCode = 1);
 void signalHandlerShutdown(int sig);
 void checkForPacketArrival(void* clientData);
+void checkInterPacketGaps(void* clientData);
 void beginQOSMeasurement();
 Boolean setupDestinationRTSPServer();
 
@@ -49,6 +53,7 @@ Medium* ourClient = NULL;
 MediaSession* session = NULL;
 TaskToken sessionTimerTask = NULL;
 TaskToken arrivalCheckTimerTask = NULL;
+TaskToken interPacketGapCheckTimerTask = NULL;
 TaskToken qosMeasurementTimerTask = NULL;
 Boolean createReceivers = True;
 Boolean outputQuickTimeFile = False;
@@ -59,6 +64,8 @@ char const* singleMedium = NULL;
 int verbosityLevel = 0;
 double endTime = 0;
 double endTimeSlop = -1.0; // extra seconds to play at the end
+unsigned interPacketGapMaxTime = 0;
+unsigned totNumPacketsReceived = ~0; // used if checking inter-packet gaps
 Boolean playContinuously = False;
 int simpleRTPoffsetArg = -1;
 Boolean sendOptionsRequestOnly = False;
@@ -76,6 +83,7 @@ unsigned short movieHeight = 180;
 unsigned movieFPS = 15;
 char* fileNamePrefix = "";
 unsigned fileSinkBufferSize = 20000;
+unsigned socketInputBufferSize = 0;
 Boolean packetLossCompensate = False;
 Boolean syncStreams = False;
 Boolean generateHintTracks = False;
@@ -92,12 +100,12 @@ struct timeval startTime;
 
 void usage() {
   *env << "Usage: " << progName
-       << " [-p <startPortNum>] [-r|-q] [-a|-v] [-V] [-e <endTime>] [-c] [-s <offset>] [-n]"
+       << " [-p <startPortNum>] [-r|-q] [-a|-v] [-V] [-e <endTime>] [-E <max-inter-packet-gap-time> [-c] [-s <offset>] [-n]"
 	   << (controlConnectionUsesTCP ? " [-t]" : "")
        << " [-u <username> <password>"
 	   << (allowProxyServers ? " [<proxy-server> [<proxy-server-port>]]" : "")
        << "]" << (supportCodecSelection ? " [-A <audio-codec-rtp-payload-format-code>|-D <mime-subtype-name>]" : "")
-       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
+       << " [-w <width> -h <height>] [-f <frames-per-second>] [-y] [-H] [-Q [<measurement-interval>]] [-F <filename-prefix>] [-b <file-sink-buffer-size>] [-B <input-socket-buffer-size>] [-m] <url> (or " << progName << " -o [-V] <url>)\n";
   //##### Add "-R <dest-rtsp-url>" #####
   shutdown();
 }
@@ -184,6 +192,14 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'E': { // specify maximum number of seconds to wait for packets:
+      if (sscanf(argv[2], "%u", &interPacketGapMaxTime) != 1) {
+	usage();
+      }
+      ++argv; --argc;
+      break;
+    }
+
     case 'c': { // play continuously
       playContinuously = True;
       break;
@@ -302,6 +318,14 @@ int main(int argc, char** argv) {
       break;
     }
 
+    case 'B': { // specify the size of input socket buffers
+      if (sscanf(argv[2], "%u", &socketInputBufferSize) != 1) {
+	usage();
+      }
+      ++argv; --argc;
+      break;
+    }
+
     // Note: The following option is deprecated, and may someday be removed:
     case 'l': { // try to compensate for packet loss by repeating frames
       packetLossCompensate = True;
@@ -380,8 +404,9 @@ int main(int argc, char** argv) {
     shutdown();
   }
 
+  // Begin by sending an "OPTIONS" command:
+  char* optionsResponse = getOptionsResponse(ourClient, url);
   if (sendOptionsRequestOnly) {
-    char* optionsResponse = getOptionsResponse(ourClient, url);
     if (optionsResponse == NULL) {
       *env << clientProtocolName << " \"OPTIONS\" request failed: "
 	   << env->getResultMsg() << "\n";
@@ -456,19 +481,36 @@ int main(int argc, char** argv) {
 			<< "-" << subsession->clientPortNum()+1 << ")\n";
 		madeProgress = True;
 
-		// Because we're saving the incoming data, rather than playing it
-		// in real time, allow an especially large time threshold (1 second)
-		// for reordering misordered incoming packets:
 		if (subsession->rtpSource() != NULL) {
-		 unsigned const thresh = 1000000; // 1 second 
-		 subsession->rtpSource()->setPacketReorderingThresholdTime(thresh);
+		  // Because we're saving the incoming data, rather than playing
+		  // it in real time, allow an especially large time threshold
+		  // (1 second) for reordering misordered incoming packets:
+		  unsigned const thresh = 1000000; // 1 second 
+		  subsession->rtpSource()->setPacketReorderingThresholdTime(thresh);
+
+		  if (socketInputBufferSize > 0) {
+		    // Set the RTP source's input buffer size as specified:
+		    int socketNum
+		      = subsession->rtpSource()->RTPgs()->socketNum();
+		    unsigned curBufferSize
+		      = getReceiveBufferSize(*env, socketNum);
+		    unsigned newBufferSize
+		      = setReceiveBufferTo(*env, socketNum, socketInputBufferSize);
+		    *env << "Changed socket receive buffer size for the \""
+			 << subsession->mediumName()
+			 << "/" << subsession->codecName()
+			 << "\" subsession from "
+			 << curBufferSize << " to "
+			 << newBufferSize << " bytes\n";
+		  }
 		}
       }
     } else {
       if (subsession->clientPortNum() == 0) {
-		*env << "No client port was specified for the \"" << subsession->mediumName()
-			<< "/" << subsession->codecName()
-			<< "\" subsession.  (Try adding the \"-p <portNum>\" option.)\n";
+	*env << "No client port was specified for the \""
+	     << subsession->mediumName()
+	     << "/" << subsession->codecName()
+	     << "\" subsession.  (Try adding the \"-p <portNum>\" option.)\n";
       } else {	
 		madeProgress = True;
       }
@@ -525,9 +567,18 @@ int main(int argc, char** argv) {
 	} else {
 	  sprintf(outFileName, "stdout");
 	}
-	FileSink* fileSink
-	  = FileSink::createNew(*env, outFileName,
-				fileSinkBufferSize, oneFilePerFrame);
+	FileSink* fileSink;
+	if (strcmp(subsession->mediumName(), "audio") == 0 &&
+	    (strcmp(subsession->codecName(), "AMR") == 0 ||
+	     strcmp(subsession->codecName(), "AMR-WB") == 0)) {
+	  // For AMR audio streams, we use a special sink that inserts AMR frame hdrs:
+	  fileSink = AMRAudioFileSink::createNew(*env, outFileName,
+						 fileSinkBufferSize, oneFilePerFrame);
+	} else {
+	  // Normal case:
+	  fileSink = FileSink::createNew(*env, outFileName,
+					 fileSinkBufferSize, oneFilePerFrame);
+	}
 	subsession->sink = fileSink;
 	if (subsession->sink == NULL) {
 	  *env << "Failed to create FileSink for \"" << outFileName
@@ -541,9 +592,9 @@ int main(int argc, char** argv) {
 			<< "\" subsession to 'stdout'\n";
 	  }
 
-	  if (strcmp(subsession->mediumName(), "video") == 0
-	      && strcmp(subsession->codecName(), "MP4V-ES") == 0
-	      && subsession->fmtp_config() != NULL) {
+	  if (strcmp(subsession->mediumName(), "video") == 0 &&
+	      strcmp(subsession->codecName(), "MP4V-ES") == 0 &&
+	      subsession->fmtp_config() != NULL) {
 	    // For MPEG-4 video RTP streams, the 'config' information
 	    // from the SDP description contains useful VOL etc. headers.
 	    // Insert this data at the front of the output file:
@@ -553,6 +604,21 @@ int main(int argc, char** argv) {
 	    struct timeval timeNow;
 	    gettimeofday(&timeNow, &Idunno);
 	    fileSink->addData(configData, configLen, timeNow);
+	    delete[] configData;
+#ifdef SUPPORT_REAL_RTSP
+	  } else if (strcmp(subsession->codecName(), "X-PN-REALAUDIO") == 0 ||
+		     strcmp(subsession->codecName(), "X-PN-REALVIDEO") == 0) {
+	    // For RealNetworks' special streams, the output file needs to begin
+	    // with a special header, in order for it to be usable.
+	    // Write this header first:
+	    unsigned headerSize;
+	    unsigned char* headerData
+	      = RealGenerateRMFFHeader(session, subsession, headerSize);
+	    struct timeval timeNow;
+	    gettimeofday(&timeNow, &Idunno);
+	    fileSink->addData(headerData, headerSize, timeNow);
+	    delete[] headerData;
+#endif
 	  }
 
 	  subsession->sink->startPlaying(*(subsession->readSource()),
@@ -662,6 +728,7 @@ void startPlayingStreams() {
 
   // Watch for incoming packets (if desired):
   checkForPacketArrival(NULL);
+  checkInterPacketGaps(NULL);
 }
 
 void tearDownStreams() {
@@ -943,6 +1010,7 @@ void shutdown(int exitCode) {
   if (env != NULL) {
     env->taskScheduler().unscheduleDelayedTask(sessionTimerTask);
     env->taskScheduler().unscheduleDelayedTask(arrivalCheckTimerTask);
+    env->taskScheduler().unscheduleDelayedTask(interPacketGapCheckTimerTask);
     env->taskScheduler().unscheduleDelayedTask(qosMeasurementTimerTask);
   }
 
@@ -969,7 +1037,7 @@ void signalHandlerShutdown(int /*sig*/) {
   shutdown(0);
 }
 
-void checkForPacketArrival(void* clientData) {
+void checkForPacketArrival(void* /*clientData*/) {
   if (!notifyOnPacketArrival) return; // we're not checking 
 
   // Check each subsession, to see whether it has received data packets:
@@ -1025,6 +1093,35 @@ void checkForPacketArrival(void* clientData) {
 			       (TaskFunc*)checkForPacketArrival, NULL);
 }
 
+void checkInterPacketGaps(void* /*clientData*/) {
+  if (interPacketGapMaxTime == 0) return; // we're not checking 
+
+  // Check each subsession, counting up how many packets have been received:
+  unsigned newTotNumPacketsReceived = 0;
+
+  MediaSubsessionIterator iter(*session);
+  MediaSubsession* subsession;
+  while ((subsession = iter.next()) != NULL) {
+    RTPSource* src = subsession->rtpSource();
+    if (src == NULL) continue;
+    newTotNumPacketsReceived += src->receptionStatsDB().totNumPacketsReceived();
+  }
+
+  if (newTotNumPacketsReceived == totNumPacketsReceived) {
+    // No additional packets have been received since the last time we
+    // checked, so end this stream:
+    *env << "Closing session, because we stopped receiving packets.\n";
+    interPacketGapCheckTimerTask = NULL;
+    sessionAfterPlaying();
+  } else {
+    totNumPacketsReceived = newTotNumPacketsReceived;
+    // Check again, after the specified delay: 
+    interPacketGapCheckTimerTask
+      = env->taskScheduler().scheduleDelayedTask(interPacketGapMaxTime*1000000,
+				 (TaskFunc*)checkInterPacketGaps, NULL);
+  }
+}
+
 // WORK IN PROGRESS #####
 class RTPTranslator: public FramedFilter {
 public:
